package org.greenstone.gsdl3.core;

import org.greenstone.gsdl3.util.*;
import org.greenstone.gsdl3.action.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

// other java classes
import java.io.File;
import java.util.HashMap;
import java.util.Enumeration;

/** core class for web interface generation. Receives requests consisting
 * of an xml representation of cgi args, and returns the page of data - in 
 * html by default. The requests are processed by the appropriate action class
 * This is a new version of the Receptionist for the nzdl interface
 * @see Action
 */
public class NZDLReceptionist extends Receptionist {

    // if we want to use different actions, overwrite this method
    //public void configure() {
    //}

    /** process - produce a page of data in response to a request 
     * if something goes wrong, it returns null - 
     * TODO:  return a suitable message to the user */
    public Element process(Element message) {

	// get the request out of the message - assume that there is only one
	Element request = (Element)GSXML.getChildByTagName(message, GSXML.REQUEST_ELEM);
	if (request == null) {
	    System.err.println("Receptionist Error: message had no request!");
	    return null; 
	}
	// check that the request type is correct
	String type = request.getAttribute(GSXML.TYPE_ATT); // returns "" if no att of this name
	if (!type.equals(GSXML.REQUEST_TYPE_CGI)) {
	    // now Receptionist forwards non-cgi requests straight to the MR, and returns the responses
	    System.err.println("Receptionist: request type is not '"+GSXML.REQUEST_TYPE_CGI+"', but it is '"+type+"', so forwarding the message to the MR!");
	    // process teh whole message - mr needs <message> tags, and 
	    // in this case, there may be more than one request in the message
	    return mr_.process(message);
	}	
	// work out which action to pass to
	String action = request.getAttribute(GSXML.ACTION_ATT);
	if (action.equals("")) {
	    System.err.println("Receptionist Error: no action specified in the request!");
	    return null;
	}
	// convert cgi short names to long names in the param list
	Element param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	if (param_list!=null) {
	    if (!cgi_.paramListToLongNames(param_list)) {
		System.err.println("Receptionist Error: couldn't convert short names to long names in the param list!");
		return null;
	    }
	}
	
	// find the  appropriate action	 - may need to load a new one
	Action a = (Action)action_map_.get(action);
	String action_name=null;
	if (a==null) { // not in the map yet
	    // try to load a new action
	    try {
		action_name = action.substring(0,1).toUpperCase()+action.substring(1)+"Action";
		Action ac = (Action)Class.forName("org.greenstone.gsdl3.action."+action_name).newInstance();
		ac.setConfigVars(config_);
		ac.setMessageRouter(mr_);
		ac.setCGI(cgi_);
		ac.configure();
		action_map_.put(action, ac);
		a = ac;
	    } catch (Exception e) {
		
		System.err.println("Receptionist Error: a new action ("+action_name+") was specified and it couldn't be created. Error message:"+e.getMessage());
		return null;
	    }
	}

	// add in the  system stuff - display and config - this all goes into 
	// a pageExtra element
	Document message_doc = message.getOwnerDocument();
	Element page_extra = message_doc.createElement(GSXML.PAGE_EXTRA_ELEM);
	message.appendChild(page_extra);
	String lang = request.getAttribute(GSXML.LANG_ATT);
	Element display = getDisplayElement(lang);
	page_extra.appendChild(message_doc.importNode(display, true));
	page_extra.appendChild(message_doc.importNode(config_.config_xml_, true));

	// create the nzdl page extra
	// check for collection
	if (param_list != null) {
	    HashMap params = GSXML.extractParams(param_list, false);
	    String collection = (String)params.get(GSCGI.COLLECTION_ARG);
	    if (collection != null && !collection.equals("")) {
		addNZDLPageExtra(page_extra, lang, collection);
	    }
	}
	    //message.appendChild(message_doc.importNode(getPageExtra(lang, collection), true));
	  
	    //} else {
	    //message.appendChild(message_doc.createElement("pageExtra"));
	    // } 
	    //} else {
	    //  message.appendChild(message_doc.createElement("pageExtra"));
	    //}
	
	Element page = a.process(message);
	return page;
	
    }

    //protected Element getPageExtra(String lang, String collection) {
    protected void addNZDLPageExtra(Element page_extra, String lang, 
				    String collection) {
	
	Document doc = page_extra.getOwnerDocument();

	// get the coll description to see whatservices are available
	Element info_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element info_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE, collection, lang);
	Element param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	info_request.appendChild(param_list);
	
	Element param = GSXML.createParameter(doc_,GSXML.SUBSET_PARAM, GSXML.SERVICE_ELEM+GSXML.LIST_MODIFIER);
	param_list.appendChild(param);
	info_message.appendChild(info_request);

	Element info_response = mr_.process(info_message);

	//System.out.println("NZDLrecept: coll response = "+converter_.getPrettyString(info_response));
	Element service_list = (Element)GSXML.getNodeByPath(info_response, "response/collection/serviceList");
	
	Element phrase_service = GSXML.getNamedElement(service_list, GSXML.SERVICE_ELEM, GSXML.NAME_ATT, "PhindApplet");
	if (phrase_service !=null) {
	    page_extra.appendChild(doc.importNode(phrase_service, false));
	}

	Element classifier_service = GSXML.getNamedElement(service_list, GSXML.SERVICE_ELEM, GSXML.NAME_ATT, "ClassifierBrowse");
	if (classifier_service == null) {
	    return; // no browsing stuff
	}

	// find the list of classifiers
	info_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	String to = GSPath.appendLink(collection, "ClassifierBrowse");
	info_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE, to, lang); 
	info_message.appendChild(info_request);
		
	info_response = (Element) mr_.process(info_message);

	classifier_service = (Element)GSXML.getNodeByPath(info_response, "response/service");
	
	page_extra.appendChild(doc.importNode(classifier_service, true));
	return;
	
	
    }

}
    
